import casadi

from .__about__ import __version__

Variable = casadi.SX.sym

from . import math, models
from ._lab_compressible import LabCompressible
from ._lab_compressible import LabCompressible as Lab
from ._lab_incompressible import LabIncompressible
from ._material import Function
from ._material import Function as FunctionScalar
from ._material import FunctionTensor
from ._material import Material
from ._material import Material as MaterialScalar
from ._material import MaterialTensor
from ._templates import (
    MaterialComposite,
    MaterialHyperelastic,
    MaterialHyperelasticPlaneStrain,
    MaterialHyperelasticPlaneStressIncompressible,
    MaterialHyperelasticPlaneStressLinearElastic,
    MaterialTensorGeneral,
    ThreeFieldVariation,
    ThreeFieldVariationPlaneStrain,
    TwoFieldVariation,
    TwoFieldVariationPlaneStrain,
)

__all__ = [
    "__version__",
]



from ..math import astensor, asvoigt, det, gradient, inv, trace


def finite_strain_viscoelastic(x, mu, eta, dtime):
    "Finite strain viscoelastic material formulation."

    # split input into the deformation gradient and the vector of state variables
    F, Cin = x[0], x[-1]

    # update of state variables by evolution equation
    Ci = astensor(Cin) + mu / eta * dtime * det(F) ** (-2 / 3) * (F.T @ F)
    Ci = det(Ci) ** (-1 / 3) * Ci

    # first invariant of elastic part of right Cauchy-Green deformation tensor
    I1 = det(F) ** (-2 / 3) * trace((F.T @ F) @ inv(Ci))

    # first Piola-Kirchhoff stress tensor and state variable
    return gradient(mu / 2 * (I1 - 3), F), asvoigt(Ci)



from . import microsphere
from ._helpers import (
    displacement_pressure_split,
    isochoric_volumetric_split,
    volumetric,
)
from ._hyperelasticity_anisotropic import fiber, fiber_family, holzapfel_gasser_ogden
from ._hyperelasticity_isotropic import (
    arruda_boyce,
    extended_tube,
    linear_elastic,
    mooney_rivlin,
    neo_hooke,
    ogden,
    saint_venant_kirchhoff,
    third_order_deformation,
    van_der_waals,
    yeoh,
)
from ._misc import morph
from ._pseudo_elasticity import ogden_roxburgh
from ._templates import Morph, NeoHookeOgdenRoxburgh, Viscoelastic
from ._viscoelasticity import finite_strain_viscoelastic
from .microsphere.nonaffine import miehe_goektepe_lulei
